<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2007 Tim Lochmueller, Sareen Millet
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(PATH_tslib.'class.tslib_pibase.php');

/**
 * Extends tslib_cObj to change the path for the images
 *
 */
class ux_tslib_cObj extends tslib_cObj {
	
	var $fl_conf = array();     // the configuration array for fl_realurl_image
	var $imgTexts = array();  	// Possible Text that can serve as a basis for file name or alternative text generation
	
	/**
	* Returns a <img> tag with the image file defined by $file and processed according to the properties in the TypoScript array.
	* Mostly this function is a sub-function to the IMAGE function which renders the IMAGE cObject in TypoScript. This function is called by "$this->cImage($conf['file'],$conf);" from IMAGE().
	*
    * @param		string		File TypoScript resource
    * @param		array		TypoScript configuration properties for the IMAGE object
    * @return		string		<img> tag, (possibly wrapped in links and other HTML) if any image found.
    * @access private
    * @see IMAGE()
    */
	function cImage($file,$conf) {           
		$info = $this->getImgResource($file,$conf['file.']);
        $GLOBALS['TSFE']->lastImageInfo = $info;
        if (is_array($info)) {
			$info[3] = t3lib_div::png_to_gif_by_imagemagick($info[3]);
        	$GLOBALS['TSFE']->imagesOnPage[]=$info[3];                  // This array is used to collect the image-refs on the page...
			 // Backwards compatible:
        	if (!strlen($conf['altText']) && !is_array($conf['altText.'])) {
        		$conf['altText'] = $conf['alttext'];
            	$conf['altText.'] = $conf['alttext.'];
        	}
        	
			###################################
			## Here begins RealUrl_image ######
			###################################
			
			// initialice class variables
			$this->init_fl_realurl_image($conf);			
			
			if (trim($this->fl_conf['fl_realurl_image.']['altText']) != "" || 
				trim($this->fl_conf['fl_realurl_image.']['fileText']) != "" ||
				trim($this->fl_conf['fl_realurl_image.']['altText.']) != "" || 
				trim($this->fl_conf['fl_realurl_image.']['fileText.']) != ""
				) {
				// get texts that contain information to the image
				$this->imgTexts = $this->getInfo4Image();
				// get a text as a basis for the file name and alternative text generation according to the hierarchy 
				$altText = $this->getBasisText("altText"); 
				$fileText = $this->getBasisText("fileText");
			}
			             
			// set the alt attribute
 			if ($altText != "") {
				$conf['altText'] = $altText;
				unset($conf['altText.']);  // std-wrap already fulfilled
			}           
			// generate the attributes for the <img>-tag
            $altParam = $this->getAltParam($conf);        
            
            // the path to the real image location
            $imgPathOrg = htmlspecialchars(trim(t3lib_div::rawUrlEncodeFP($info[3]))); // true path to image: typo3temp/pics/2305e38d9c.jpg
			// realurl_image gets in action
			if ($imgPathOrg != "" && $fileText != "") {
                // generate a RealUrl-filename for the picture and set the pair picture - realUrl in the DB
				$imgPathNew = $this->makeRealURLImgPath($this->encodeTitle($fileText),$imgPathOrg);
			}
			// realurl_image sleeps => nothing happens
			else {
			    $imgPathNew = $imgPathOrg;
			}

            // generate the <img>-tag
            $theValue = '<img src="'
						.htmlspecialchars($GLOBALS['TSFE']->absRefPrefix).$imgPathNew.'"'
			            .' width="'.$info[0].'" height="'.$info[1].'"'
						.$this->getBorderAttr(' border="'.intval($conf['border']).'"')
						.($conf['params']?' '.$conf['params']:'').($altParam)
						.' />';
			##################################
        	### Here ends RealURL_Image ######
        	##################################
        	
            if ($conf['linkWrap']) {
            	$theValue = $this->linkWrap($theValue,$conf['linkWrap']);
            } 
			elseif ($conf['imageLinkWrap']) {
        		$theValue = $this->imageLinkWrap($theValue,$info['origFile'],$conf['imageLinkWrap.']);
            }
			return $this->wrap($theValue,$conf['wrap']);
    	}
	}
	
	###########################################################################################################
	## Helper functions #######################################################################################
	###########################################################################################################
	/*
     * initalise the global var $this->fl_conf	 
     * Setting prioritys:
     * 1) IMAGE-Object of requested image
     * 2) TS:config.fl_realurl_image from e.g. ext_typoscript_setup.txt
     * 3) $GLOBALS['fl_realurl_image'] from ext_conf_template.txt   
     *
     * @param		nothing     
     * @return 		nothing
     */
	function init_fl_realurl_image($conf) {	
			#$GLOBALS["TSFE"]->set_no_cache();	            // set this for debugging, but turn it off when go to beta test
	    	
        	// configuration for fl_realurl_image via ext_conf_template.txt and ext_typoscript_setup.txt
      		$global_conf = t3lib_div::array_merge_recursive_overrule($GLOBALS['fl_realurl_image'],$GLOBALS['TSFE']->tmpl->setup['config.']['fl_realurl_image.'],0,0);  
			// fill up this global configuration into the conf of the IMAGE Object
			if (!$conf['fl_realurl_image.']) $conf['fl_realurl_image.'] = array();
			$conf['fl_realurl_image.'] = t3lib_div::array_merge_recursive_overrule($global_conf,$conf['fl_realurl_image.']);
			
			// correcting input mistaces
			if (strstr($conf['fl_realurl_image.']['fileName'],".")) {
				$temp_array = explode(".",$conf['fl_realurl_image.']['fileName']);
				$conf['fl_realurl_image.']['fileName'] = $temp_array[0];
			}
			if (!$this->fl_conf['fl_realurl_image.']['spaceCharacter']) {
				$this->fl_conf['fl_realurl_image.']['spaceCharacter'] = "-";
			}
			
			// setting global var:
			$this->fl_conf = $conf;
	}	

	/*
     * Get a basis text for file name or alternative text generation
     *    
     * @param		string		"fileText" or "altText" 
     * @return 		string		a text to the image
     */	
	function getBasisText ($key) {
		if (trim($this->fl_conf['fl_realurl_image.'][$key]) != "" && !$this->fl_conf['fl_realurl_image.'][$key.'.']) {
			if (strstr($this->fl_conf['fl_realurl_image.'][$key],"hierarchy:")) {
				list($waste,$hierarchy) = explode(":",$this->fl_conf['fl_realurl_image.'][$key]);
				$fileText =  $this->getHierarchyText($hierarchy);
			}
			else {
				$fileText =  $this->fl_conf['fl_realurl_image.'][$key];
			}
			$fileText =  trim($this->stdWrap($fileText,$this->fl_conf['fl_realurl_image.'][$key.'.']));
        }
        return $fileText;
	}
	
	/*
     * Get according to a pipe separated hierarchy list one text to the image out of an array
     *    
     * @param		string		pipe separated hierarchy list    
     * @return 		string		a text describing text to a image
     */	
	function getHierarchyText($hierarchy) {
		$hierarchyArray = explode("|",$hierarchy);		
		foreach($hierarchyArray as $key) {
			$key = trim($key);
			if (trim($this->imgTexts[$key]) != "") {
				return $this->imgTexts[$key];
			}
		}
		return "";
	}
		
	/*
     * Get texts that contain information to an image
     * 
     * $this->fl_conf: fileNameText, defaultFileNameText, (file, altText,titleText,longdescURL)     
     *
     * @return 		array		texts that contain information to an image
     */
	function getInfo4Image() {
		$info = array();
		
		// file_name
		$file = $this->getImgResource($this->fl_conf['file'],$this->fl_conf['file.']);   // $this->cObj = $this as we XCLASS cObj	
		$info['file_name'] = basename($file['origFile']);
		
        // DAM info to the image: title, file_name, description, ...
        $damInfo = $this->getDAMinfo($info['file_name']);
        if (is_array($damInfo)) {
        	$info = array_merge($info,$damInfo);
        }
        
        // TS defined texts from fl_realurl_image.addText.: default, what_ever_is_defined, ...
        if (is_array($this->fl_conf['fl_realurl_image.']['addText.'])) {
        	foreach($this->fl_conf['fl_realurl_image.']['addText.'] as $key => $val) {
        	    if (!strstr($key,".") || (strstr($key,".") && !$this->fl_conf['fl_realurl_image.']['addText.'][$key])){
        			$info[$key] = trim($this->stdWrap($this->fl_conf['fl_realurl_image.']['addText.'][$key], $this->fl_conf['fl_realurl_image.']['addText.'][$key.'.']));
        		}
        	}
        }
        
		// IMAGE-Object Info: altText, titleText, longdescURL
		$info['altText'] = trim($this->stdWrap($this->fl_conf['altText'], $this->fl_conf['altText.']));
		$info['titleText'] = trim($this->stdWrap($this->fl_conf['titleText'],$this->fl_conf['titleText.']));
		$info['longdescURL'] = trim($this->stdWrap($this->fl_conf['longdescURL'],$this->fl_conf['longdescURL.']));
		
		// title of the page
		$info['pageTitle'] = $this->getPageTitle(); 
		
		// make readable text
		if ($info['file_name']) {
			$info['file_name'] = $this->fileName2text($info['file_name']);
		}
		if ($info['file_dl_name']) {
			$info['file_dl_name'] = $this->fileName2text($info['file_dl_name']);
		}

		// return something informative to the image
		return $info;
	}
	
	/**
	 * get information to an image from DAM
	 * title, description, keywords, file_dl_name, ... any field in table tx_dam	 
	 * 
	 * @return		array		the title (or something informative) to the image from DAM
	 */  
	function getDAMinfo($fileName) { 
	    // DAM not installed
		if (!t3lib_extMgm::isLoaded('dam')) {
			return;  
		} 

		// get information for item from tx_dam
		$items =  $GLOBALS['TYPO3_DB']->exec_SELECTgetRows(
			'*',
			'tx_dam',
			'(file_name = "'.$fileName.'") '
			.$this->enableFields('tx_dam')   							// $this->cObj = $this as we XCLASS cObj	
			.' AND sys_language_uid=0'
    	);  		  
    	// get language overlay
    	if ($GLOBALS['TSFE']->sys_language_content) {
			$OLmode = ($GLOBALS['TSFE']->sys_language_mode == 'strict'?'hideNonTranslated':'');
			for ($i = 0; $i < sizeof($items); $i++) {
				$items[$i] = $GLOBALS['TSFE']->sys_page->getRecordOverlay('tx_dam', $items[$i], $GLOBALS['TSFE']->sys_language_content, $OLmode);
			}
		}
		
		// no information for image in DAM stored
		if (!is_array($items[0])) {
			return;
		}
		
		// return array with information
		return $items[0];
 	}

	/*
     * Get the title of the page for the current language
     *
     * @return String Pagetitle
     */
	function getPageTitle() {
        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*','pages','uid='.$GLOBALS['TSFE']->id);
        $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
        $title = $row['title']; 
        if ($GLOBALS['TSFE']->sys_language_content) {
            $OLmode = ($GLOBALS['TSFE']->sys_language_mode == 'strict'?'hideNonTranslated':'');
            $row = $GLOBALS['TSFE']->sys_page->getRecordOverlay('pages_language_overlay', $row, $GLOBALS['TSFE']->sys_language_content, $OLmode);
            if(isset($row['title'])) {
				$title = $row['title'];
			}
        }
        return $title;
    }

	/**
     * Convert a filename to something that is more human readable
     *
     * @param        string                filename or title
     * @return       string               title
     */        
	function fileName2text($text) {
		if (strstr($text,".")) {
			$temp = explode(".",$text);
			$text = $temp[0];
		}
		$text = urldecode($text);
		$text = str_replace("-"," ",$text);
		$text = str_replace("_"," ",$text);
		return $text;
	}

	/**
     * Convert a title to something that can be used in an page path:
     * - Convert spaces to underscores
     * - Convert non A-Z characters to ASCII equivalents
     * - Convert some special things like the 'ae'-character
     * - Strip off all other symbols
     * Works with the character set defined as "forceCharset"
     * 
     * $this->fl_conf: spaceCharacter,encodeTitle_userProc
     *
     * @param        string                Input title to clean
     * @return       string                Encoded title, passed through rawurlencode() = ready to put in the URL.
     * @see rootLineToPath()
     */
	function encodeTitle($title) {
    	// Fetch character set:
        $charset = $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] ? $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] : $GLOBALS['TSFE']->defaultCharSet;
		
		// Convert to lowercase:
        $processedTitle = $GLOBALS['TSFE']->csConvObj->conv_case($charset,$title,'toLower');
		
		// Convert some special tokens to the space character:
        $space = $this->fl_conf['fl_realurl_image.']['spaceCharacter'];
        $processedTitle = strtr($processedTitle,' -+_',$space.$space.$space.$space);        // convert spaces
		
		// Convert extended letters to ascii equivalents:
        $processedTitle = $GLOBALS['TSFE']->csConvObj->specCharsToASCII($charset,$processedTitle);
		
		// Strip the rest...:
        $processedTitle = ereg_replace('[^a-zA-Z0-9\\'.$space.']', $space, $processedTitle);  // replace other characters with $space
        $processedTitle = ereg_replace('\\'.$space.'+',$space,$processedTitle);               // Convert multiple 'spaces' to a single one
        $processedTitle = trim($processedTitle,$space);
		
		// user procession function for encoding the title
        if ($this->fl_conf['fl_realurl_image.']['encodeTitle_userProc'])        {
        	$params = array(
            	'pObj' => &$this,
                'title' => $title,
                'processedTitle' => $processedTitle,
            );
            $processedTitle = t3lib_div::callUserFunction($this->fl_conf['fl_realurl_image.']['encodeTitle_userProc'], $params, $this);
        }
        
		// Return encoded URL:
        return rawurlencode($processedTitle);
	}
	    
    /**
     * Generate the RealUrl Path to an image:
     *
     * $this->fl_conf: appendHash, imageFolder
     * 	      
     * @param string The encoded base URL for this image
     * @param string orginal path to picture z.B. typo3temp/pics/2305e38d9c.jpg
     * @return string The ready RealUrl Path for the image
     * @see rootLineToPath()
     */
    function makeRealURLImgPath($realurl_base,$img_path) {
    
        $realurl_post = 0; // a postfix to get a unique realUrl
        list($img_base,$realurl_end) = explode('.',basename($img_path));
        $realurl_end = ".".$realurl_end;
        
        // if there is nothing sensible for a filename use a default (alternative: "return $img_path;" could make sense)
        if ($realurl_base == '') {
			$realurl_base = $img_base;
		}
		
		// base for the realurl file name
        if ($this->fl_conf['fl_realurl_image.']['appendHash']) {
			if (!is_numeric($this->fl_conf['fl_realurl_image.']['appendHash'])) {
				$this->fl_conf['fl_realurl_image.']['appendHash'] = 2;
			}
			elseif ($this->fl_conf['fl_realurl_image.']['appendHash'] > strlen($img_base)) {
				$this->fl_conf['fl_realurl_image.']['appendHash'] = strlen($img_base);
			}        
        	$hash = substr($img_base, 0, $this->fl_conf['fl_realurl_image.']['appendHash']);  
        	$realurl_base = substr($realurl_base, 0, 210-($this->fl_conf['fl_realurl_image.']['appendHash']+1)).$this->fl_conf['fl_realurl_image.']['spaceCharacter'].$hash;
        }
        else {
        	$realurl_base = substr($realurl_base, 0, 210);
        }
		
		// add the folder to the realurl file name base
        if ($this->fl_conf['fl_realurl_image.']['imageFolder'] != '')
            $realurl_base = $this->fl_conf['fl_realurl_image.']['imageFolder'].'/'.$realurl_base;
		
		// search for an empty realurl file name and write in DB	        
        do {     
            //echo $realurl_base;
            $imageRealURL = ($realurl_post == 0)?$realurl_base.$realurl_end:$realurl_base.$this->fl_conf['fl_realurl_image.']['spaceCharacter'].$realurl_post.$realurl_end;
            
            $res_realurl = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid,image_path','tx_flrealurlimage_cache','realurl_path=\''.$imageRealURL.'\'');
            $num_realurl = $GLOBALS['TYPO3_DB']->sql_num_rows($res_realurl);
            $data_realurl = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res_realurl);
            
			// realurl not yet taken by a picture 		=> take it
            if ($num_realurl == 0){             
                $insertArray = array(
                    'crdate' => time(), 
                    'tstamp' => time(),
                    'image_path' => $img_path,
                    'realurl_path' => $imageRealURL,
                    'pid' => intval($this->conf['fl_realurl_image.']['recordStorage']),
                    'page_id' => $GLOBALS['TSFE']->id
                );       
                $res_realurl = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_flrealurlimage_cache', $insertArray);
        		// create a static copy of the image
        		if ($this->fl_conf['fl_realurl_image.']['staticFileCache'] == true) {
        			$imgFolderNew = str_replace('/'.basename($imageRealURL),'',$imageRealURL);
					if (!is_dir($imgFolderNew)) {
				    	mkdir ($imgFolderNew);
					}
					if (!file_exists($imageRealURL)) {
						# original image needs to stay cause eacher
						# - it is a original image e.g. from fileadmin
						# - the images in typo3temp are necessary cause T3 gets sice and existance from there
						copy($img_path,$imageRealURL);
					}
				}
                break;
            } 
            // realurl taken by this picture 		=> take it again, but set tstamp new
			elseif ($num_realurl == 1 && $data_realurl['image_path'] == $img_path) {
			    # ToDo: set new tstamp
        		// create a static copy of the image
        		if ($this->fl_conf['fl_realurl_image.']['staticFileCache'] == true) {
        			$imgFolderNew = str_replace('/'.basename($imageRealURL),'',$imageRealURL);
					if (!is_dir($imgFolderNew)) {
				    	mkdir ($imgFolderNew);
					}
					if (!file_exists($imageRealURL)) {
						# original image needs to stay cause eacher
						# - it is a original image e.g. from fileadmin
						# - the images in typo3temp are necessary cause T3 gets sice and existance from there
						copy($img_path,$imageRealURL);
					}
				}
                break;
			}
            // realurl taken by another picture		=> look for free RealURL
			else {
                $realurl_post++;
			}
                
        } while ($num_realurl == 1 && $data_realurl['image_path'] != $img_path);
        
        // return the realurl 
        return $imageRealURL;
    } 
	
}
?>